<?php
/**
 * @file mod_roomReservationRoomWhitelistListBox.inc
 * List box that shows the rooms who can be booked
 * @author Roland Hieber (roland.hieber@wilhelm-gym.net)
 * @date 24.07.2008
 * 
 * Copyright © 2007 Roland Hieber
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

require_once("mod_roomReservationControl.inc");
require_once("mod_roomReservationRoomsManager.inc");

/*****************************************************************************/
/**
 * @page roomwhitelistlistbox_actions Actions of a 
 * mod_roomReservationRoomWhitelistListBox instance
 * @{
 * The following constants describe the actions that a
 * mod_roomReservationRoomWhitelistListBox instance can handle. They are used
 * in processRequestVariables() to determine the action that should be done
 * when the control is shown.
 */
/** Show the control (default action) */
define("MOD_ROOM_RESERVATION_RWLB_ACTION_SHOW", 0);
/** Show the addition form */
define("MOD_ROOM_RESERVATION_RWLB_ACTION_ADD", 1);
/** Process the addition form */
define("MOD_ROOM_RESERVATION_RWLB_ACTION_SUBMITADD", 2);
/** Show the deletion form */
define("MOD_ROOM_RESERVATION_RWLB_ACTION_DELETE", 3);
/** Process the deletion form */
define("MOD_ROOM_RESERVATION_RWLB_ACTION_SUBMITDELETE", 4);
/** @} */

/** @todo document */
class mod_roomReservationRoomWhitelistListBox 
  extends mod_roomReservationControl {
  /**
   * (array of integers) OIDs of the rows in the rooms table that were 
   * selected (POST data)
   */
  protected $anPostSelection = array();
  /** (constant) Display mode, see @ref roomwhitelistlistbox_actions */
  protected $cMode;
  
  protected function processRequestVariables() {
    // default values
    $this->cMode = MOD_ROOM_RESERVATION_RWLB_ACTION_SHOW;
    
    // POST data
    if(isset($_POST["mod_roomReservationRoomWhitelistListBox"])) {
      $aPost = $_POST["mod_roomReservationRoomWhitelistListBox"];
      // mode
      if(isset($aPost["action"])) {
        if(isset($aPost["action"]["add"])) {
          if($aPost["action"]["add"] == _("Add")) {
            $this->cMode = MOD_ROOM_RESERVATION_RWLB_ACTION_ADD;
          } elseif($aPost["action"]["add"] == _("OK")) {
            $this->cMode = MOD_ROOM_RESERVATION_RWLB_ACTION_SUBMITADD;
          }
        } elseif(isset($aPost["action"]["delete"])) {
          if($aPost["action"]["delete"] == _("Delete")) {
            $this->cMode = MOD_ROOM_RESERVATION_RWLB_ACTION_DELETE;
          } elseif($aPost["action"]["delete"] == _("OK")) {
            $this->cMode = MOD_ROOM_RESERVATION_RWLB_ACTION_SUBMITDELETE;
          }
        }
      }
      // selection
      if(isset($aPost["l"])) {
        foreach($aPost["l"] as $nOid => $bChecked) {
          if($bChecked) {
            $this->anPostSelection[] = $nOid;
          }
        }
      }
    }
    
    // process the forms
    if($this->cMode == MOD_ROOM_RESERVATION_RWLB_ACTION_SUBMITADD) {
      $h = db_query("SELECT name FROM rooms WHERE oid IN ".
        qdb_arr($this->anPostSelection));
      while($a = pg_fetch_array($h)) {
        $this->oCfg->whitelistRoom($a["name"]);
      }
    }

    if($this->cMode == MOD_ROOM_RESERVATION_RWLB_ACTION_SUBMITDELETE) {
      $h = db_query("SELECT name FROM rooms WHERE oid IN ".
        qdb_arr($this->anPostSelection));
      while($a = pg_fetch_array($h)) {
        $this->oCfg->unWhitelistRoom($a["name"]);
      }
    }
  }
  
  protected function doShow() {
    echo "<form method='post'>";
    TreeView(array(_("Room")));
    switch($this->cMode) {
      case MOD_ROOM_RESERVATION_RWLB_ACTION_DELETE:
        $this->showDeleteForm();
        break;
      case MOD_ROOM_RESERVATION_RWLB_ACTION_ADD: $this->showAddForm(); break;
      default:
      case MOD_ROOM_RESERVATION_RWLB_ACTION_SHOW: $this->showForm(); break;
    }
    _TreeView();
    echo "</form>\n";
  }

  /**
   * Print the form if not delete nor add was requested
   * @return void
   */
  protected function showForm() {
    $aoRooms = $this->oCfg->getWhitelistedRooms();
    // only show add button if there are still some unlisted rooms
    if(count(mod_roomReservationRoomsManager::getRooms()) > count($aoRooms)) {
      TreeViewLine(sprintf("<%s name='mod_roomReservationRoomWhitelistListBox".
        "[action][add]' value='%s' />", $GLOBALS["stdbtn"], _("Add")));
    }
    $this->showList($aoRooms);
    // toolbar
    printf("<tr><td class='tbbtm' colspan='%d'>", $GLOBALS["treeview_cols"]);
    CheckCombo();
    printf("<%s name='mod_roomReservationRoomWhitelistListBox[action]".
    	"[delete]' value='%s' />", $GLOBALS["stdbtn"], _("Delete"));
    echo "</td></tr>\n";
  }
  
  /**
   * Print the addition form
   * @return void
   */
  protected function showAddForm() {
    // only list rooms that are not already whitelisted
    $aoRooms = array_diff(mod_roomReservationRoomsManager::getRooms(),
      $this->oCfg->getWhitelistedRooms());
    TreeViewSubtitle(_("Add"));
    $this->showList($aoRooms);
    TreeViewLine(sprintf("<p><%s name='mod_roomReservationRoomWhitelistList".
    	"Box[action][add]' value='%s' /> <%s name='mod_roomReservationRoom".
    	"WhitelistListBox[action][add]' value='%s' /></p>", $GLOBALS["stdbtn"],
       _("OK"), $GLOBALS["stdbtn"], _("Cancel")));
  }
  
  /**
   * Show the deletion form
   * @return void
   */
  protected function showDeleteForm() {
    // list rooms in selection
    $aoRooms = array();
    $h = db_query("SELECT name FROM rooms WHERE oid IN ".
      qdb_arr($this->anPostSelection));
    foreach($this->anPostSelection as $nOid) {
      $aoRooms[] = mod_roomReservationRoomsManager::getRoomByOid($nOid);
    }
    TreeViewSubtitle(sprintf(_("Following %s will be deleted"),
      _c("room-reservation:rooms")));
    $this->showList($aoRooms, false);
    TreeViewLine(sprintf("<p><%s name='mod_roomReservationRoomWhitelistList".
    	"Box[action][delete]' value='%s' /> <%s name='mod_roomReservationRoom".
    	"WhitelistListBox[action][delete]' value='%s' /></p>",
       $GLOBALS["stdbtn"], _("OK"), $GLOBALS["stdbtn"], _("Cancel")));
  }
  
  /**
   * Show the list items
   * @param $aoRooms (array of mod_roomReservationRoom objects) List items
   * @param $bCheckboxes (bool) Whether to show checkboxes
   */
  protected function showList($aoRooms, $bCheckboxes = true) {
    if(count($aoRooms) > 0) {
      foreach($aoRooms as $o) {
        // fetch oid from SQL table
        $nOid = pg_fetch_result(db_query("SELECT oid FROM ".
          "rooms WHERE name = $1", $o->getName()), 0, "oid");
        $sBox = $bCheckboxes ? sprintf("<%s id='box%d' name='mod_room".
        	"ReservationRoomWhitelistListBox[l][%d]' value='1'%s /><label ".
        	"for='box%d'>%s%s</label>", $GLOBALS["smlchk"], $nOid, $nOid,
          @$this->anPostSelection[$nOid] ? " checked='checked'" : "", $nOid,
          icon("host"), $o->getName()) : 
          sprintf("<input type='hidden' name='mod_roomReservationRoom".
        	"WhitelistListBox[l][%d]' value='1' />%s%s", $nOid, icon("host"),
          $o->getName());
        TreeViewLine($sBox);
      }
    } else {
      TreeViewEmpty();
    }
  }
}
?>
